/**
 * Copyright (c) 2007-2015, Kaazing Corporation. All rights reserved.
 */

package com.kaazing.gateway.client;

import com.google.gwt.core.client.JavaScriptObject;

public final class WebSocketFactory extends JavaScriptObject {

    protected WebSocketFactory() {}
    
     /**
     * Creates a new WebSocketFactory instance.
     */
    public static native WebSocketFactory create() /*-{
    	if(typeof($wnd.WebSocketFactory) == "function") {
        	return new $wnd.WebSocketFactory();
        }
        else {
        	return null;
        }
    }-*/;
    
	/**
     * get the WebSocketExtension with the extension name.
     */
    public native WebSocketExtension getExtension(String name) /*-{
		return this.getExtension(name);
	}-*/;
    
	/**
     * add or update the WebSocketExtension to this WebSocketFactory object.
     */
    public native void setExtension(WebSocketExtension extension) /*-{
		this.setExtension(extension);
	}-*/;
    
    /**
     * set a ChallengeHandler for this WebSocketFactory, all WebSocket connections created by this factory will use the challengeHandler
     * to login.
     */
    public native void setChallengeHandler(JavaScriptObject challengeHandler) /*-{
    	this.setChallengeHandler(challengeHandler);
	}-*/;
    
    /**
     * Returns the default connection timeout in milliseconds.
     * 
     * @return int default connection timeout
     */
    public native int getDefaultConnectTimeout() /*-{
        return this.getDefaultConnectTimeout();
    }-*/;

    /**
     * Sets the default connection timeout in milliseconds.
     * 
     * @param timeout    default connection timeout in milliseconds
     */
    public native void setDefaultConnectTimeout(int timeout) /*-{
        return this.setDefaultConnectTimeout(timeout);
    }-*/;

    /**
     * Creates a new WebSocket connection with url and protocol.
     * @throws WebSocketException 
     */
    public WebSocket createWebSocket(String url, String[] protocols) throws WebSocketException {
    	return new WebSocket(this, url, protocols);
	};
}
